/* 
 *  Ths code in this file is part of tcptrack. For more information see
 *    http://www.rhythm.cx/~steve/devel/tcptrack
 *
 *     Copyright (C) Steve Benson - 2003
 *
 *  tcptrack is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your
 *  option) any later version.
 *   
 *  tcptrack is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  
 */
#define _BSD_SOURCE 1
#define _REENTRANT
#include <queue>
#include <time.h>
#include <stdio.h>
#include <pthread.h>
#include "headers.h"
#include "TCContainer.h"
#include "PacketBuffer.h"

PacketBuffer::PacketBuffer(TCContainer *nc)
{
	inq = &pq1;
	outq = &pq2;
	c=nc;

	//
	// Start up maintenence thread
	//
	int rc = pthread_create(&maint_thread_tid,NULL,pbmaint_thread_func,this);
	if( rc )
	{
		perror("pthread_create");
		exit(1); 
	}

}

void PacketBuffer::pushPacket( const u_char * packet )
{
	/* copied & pasted from Tim Carsten's pcap tutorial */
	const struct sniff_ethernet *ethernet; /* The ethernet header */
	const struct sniff_ip *ip; /* The IP header */
	const struct sniff_tcp *tcp; /* The TCP header */
	const u_char *payload; /* Packet payload */
	/* For readability, we'll make variables for the sizes of each of the structures */
	int size_ethernet = sizeof(struct sniff_ethernet);
	int size_ip = sizeof(struct sniff_ip);
	int size_tcp = sizeof(struct sniff_tcp);

	// parse the headers off the packet 
	ethernet = (struct sniff_ethernet*)(packet);
	ip = (struct sniff_ip*)(packet + size_ethernet);
	tcp = (struct sniff_tcp*)(packet + size_ethernet + size_ip);
	payload = (u_char *)(packet + size_ethernet + size_ip + size_tcp);

	// copy from pcap's buffer into a new buffer
	const u_char *newpkt;
	int newpkt_len = size_ethernet+size_ip+size_tcp+ntohs(ip->ip_len);
	//fprintf(stderr,"allocating space for new packet, %d\n",newpkt_len);
	/* the life of the mem pointed to by newpkt is as follows:
	 * PacketBuffer hands it off to TCContainer::processPacket
	 * if a TCPConnection accepts the packet (or is created by it), 
	 *  the TCPConnection will be responsible for freeing it
	 * if it is not accepted by any TCPConnection, then TCPConnection
	 *  will free it.
	 */
	newpkt = (const u_char *) malloc(newpkt_len); 
	memcpy((void *)newpkt,packet,newpkt_len);

	pthread_mutex_lock(&inq_lock);
	inq->push(newpkt);
	pthread_mutex_unlock(&inq_lock);
}

void PacketBuffer::maint_thread_run()
{
	struct timespec t;
	const u_char *packet;
	
	while(true)
	{
		t.tv_sec=0;
		t.tv_nsec=100000;
		nanosleep(&t,NULL);
		
		//fprintf(stderr,"PacketBuffer::maint_thread_run: running\n");
		
		// swap in & out queues
		// this allows the input queue to be unlocked ASAP so the Sniffer
		// won't be delayed for too long.
		pthread_mutex_lock(&inq_lock);
		if( inq == &pq1 )
		{
			inq = &pq2;
			outq = &pq1;
		}
		else
		{
			inq = &pq1;
			outq = &pq2;
		}
		pthread_mutex_unlock(&inq_lock);
		
		// process outq
		packet=NULL;
		while( ! outq->empty() )
		{
			packet=NULL;
			packet=outq->front();
			c->processPacket(packet);
			outq->pop();
		}
	}

}

////////////////////////

void *pbmaint_thread_func( void *arg )
{
	PacketBuffer *pb = (PacketBuffer *) arg;
	pb->maint_thread_run();
	return NULL;
}
